unit MessageDialogBx;
(* A more flexible version of standard message dialog boxes

   ########### NOTES ######################################################################
   ##                                                                                    ##
   ##  Dates are always in ddmmyy format (Americans nay need to fiddle)                 ##
   ##                                                                                    ##
   ##  Feel free to hack or use in any way                                              ##
   ##                                                                                    ##
   ##  Originated by Peter Fox                peter@eminent.demon.co.uk                 ##
   ##                                                                                    ##
   ##  Source is freely available at          www.eminent.demon.co.uk/delphi.htm        ##
   ##                                                                                    ##
   ##  Please drop me a line if you find it useful                                      ##
   ##                                                                                    ##
   ##  Comments,corrections and additions are all welcome                               ##
   ##                                                                                    ##
   ##  This version was put to bed on 26th Dec 2001                                     ##
   ##                                                                                    ##
   ########################################################################################


   The message is split by vertical bars into lines
   Note.  There is no wrapping of text so split lines in your code
          eg OkBx('The following file was missing|'+fname);

   The size of the box adapts to suit

   The same underlying mechanism is used for OK and Yes/No

   Optionally a help context is added

   An AlertBx is like an OKBx but with a flashing yellow background

   Optionally basic validation can be done inside the control.  The following types are supported:-
                 mdmVTAnyStr,      any input including blank string
                 mdmVTSomeStr,     non-blank string
                 mdmVTPickStr,     input must match one of the items in the default (split by bars)
                                     eg  strInputBx('Directions','Which way now?','N|E|S|W|Up|Down')
                                     The default is the first element in the list
                                     Use the down arrow to navigate or initial letters
                 mdmVTint,         Any integer
                 mdmVTPosInt,      Positive integer
                 mdmVTPoZint,      positive or zero integer
                 mdmVTFloat,       Any float (integers allowed)
                 mdmVTPosFloat,    positive float
                 mdmVTPoZfloat,    positive or zero float
                 mdmVTDy,          Any date
                 mdmVTPastDy,      Today or earlier
                 mdmVTFutureDy,    Today or later
                 mdmVTEarlierDy,   same or later than default

    *****************************************************************************
    *****************************************************************************
    *****************************************************************************
    ****                                                                     ****
    **** MESSAGE LINE BREAKS ARE INDICATED BY VERTICAL BARS                  ****
    **** eg Okbx('The following file is missing|'+fname+'|Try another one'); ****
    ****                                                                     ****
    *****************************************************************************
    *****************************************************************************
    *****************************************************************************

   
                                                                                         
     I don't know if this convention is suitable for you but I use an OK box             
     to indicate something happening, some information or some problem that the user     
     could experience any day.  I save ALERTs for when they are likely to need support   
     or scour the very back pages of the manuals and then proceede with extreme caution. 
                                                                                         
     Couldn't create invoice - No items to put on it (we may have deleted them)..... OK  
     Couldn't create invoice - Duplicate invoice number (how did THAT happen!)... ALERT  
                                                                                         
   

   
                                                                                         
     I have assumed that you're not likely to need help buttons on inputs as you'd       
     put the pertinent information into the blurb.  If you want it though you can        
     go to the underlying procedures.                                                    
                                                                                         
   

   
                                                                                         
     GETTING GOING                                                                       
     (Method a) - Simply add to your project as just another unit                        
     (Method b) - Create a new package (or an existing one)                              
                  Add this to the package                                                
                  Compile the package                                                    
                  Install the package                                                    
                                                                                         
   

   
                                                                                         
    See the postscript for a 2 minute tutorial on making your own dialogs                
                                                                                         
   



   (1) Install package
   Note.  There is no component to put onto a form using the VCL.

   (2) Add "Uses MessageDialogBx" where appropriate

   (3) Call OKBx(msg) for information and run of the mill "you can't do that there 'ere" messages

       Call AlertBx(msg) for system problems where user needs to address some fundamental problem
                         which is likely to need expert assistance or an in-depth read of the manual

       Call YesNoBx(msg) to get a confirmation/rejection
       Call YesNoBx(msg,yesbutton,nobutton) to display customised buttons

       Call MsgBx for roll-your-own and additional help context button

       Call StrInputBx as a replacement for InputBox

       call StrQueryBx as a replacement for InputQuery

       call StrBx as a replacement for InputQuery but relying on Return and Escape to do OK/Cancel



       try the following in the code of a button click and use initial letters and down arrow
       |
       |  v := 'North|South|East|West';
       |  if Strquerybx('Move','Which direction (N-S-E-W)|do you want to go',v,mdmvtPickStr) then begin
       |    caption := v;
       |  end else begin
       |    caption := ':( '+v;
       |  end;




*)





interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TMdMode = (mdmOK,mdmAlert,mdmQues,mdmInput);

  TMdmValType = (// strings
                 mdmVTAnyStr,     // any input including blank string
                 mdmVTSomeStr,    // non-blank string
                 mdmVTPickStr,    // input must match one of the items in the default (split by bars)
                                  // eg  strInputBx('Directions','Which way now?','N|E|S|W|Up|Down')
                                  // The default is the first element in the list
                                  // Use the down arrow to navigate or initial letters
                 // integer
                 mdmVTint,        // Any integer
                 mdmVTPosInt,     // Positive integer
                 mdmVTPoZint,     // positive or zero integer
                 // float (could be used for currency if signs left out)
                 mdmVTFloat,      // Any float (integers allowed)
                 mdmVTPosFloat,   // positive float
                 mdmVTPoZfloat,   // positive or zero float
                 // date  ddmmyy format or B=Beg-of-Time E=End-of-time T=Today N-Not Valid
                 mdmVTDy,         // Any date
                 mdmVTPastDy,     // Today or earlier
                 mdmVTFutureDy,   // Today or later
                 mdmVTEarlierDy,  // Same or earlier than default
                 mdmVTLaterDy);   // same or later than default

  TfMessageDialogBx = class(TForm)
    bu1: TButton;
    bu2: TButton;
    buHelp: TButton;
    Timer: TTimer;
    eb: TEdit;
    procedure FormPaint(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure buHelpClick(Sender: TObject);
    procedure ebKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ebChange(Sender: TObject);
    procedure ebKeyPress(Sender: TObject; var Key: Char);
  private
    fmode        : TmdMode;        // OK-Alert-Input
    fmsg         : string;         // Blurb
    fHelpContext : THelpContext;   // link to application help file
    fValtype     : TMdmValType;    // are we validating the input in any way
    fStartVal    : string;         // what was the original value
    fStartDate   : Tdatetime;      // possibly a default date to compare with input
    function IsInputOK:boolean;
    function chop(var Mainstring : string; const Chopper : string):string;
    function Pickstrval:string;
    function ExtendPick(const txt:string):string;
    function Decodeddmmyy(const text:string;var d:tdatetime):boolean;
    function NextStrVal:string;
    function Execute(const BoxMode:TmdMode;
                     const msg,b1caption,b2caption:string;
                     const hlpctxt:THelpContext;
                     var   value:string;
                     const valType:TMdmValType):Boolean;
  end;


  function MsgBxDlgExecute(const BoxMode:TmdMode;
                           const msg,b1caption,b2caption:string;
                           const hlpctxt:THelpContext):Boolean;   overload;

  function MsgBxDlgExecute(const BoxMode:TmdMode;
                           const msg,b1caption,b2caption:string;
                           const hlpctxt:THelpContext;
                           var   value:string):Boolean;           overload;

  function MsgBxDlgExecute(const BoxMode:TmdMode;
                           const msg,b1caption,b2caption:string;
                           const hlpctxt:THelpContext;
                           var   value:string;
                           const valType:TMdmValType):Boolean;           overload;

  //============== these are the calls =======================================
  procedure OkBx(const msg:string);                                overload;
  procedure OkBx(const msg:string; const hlpctxt:THelpContext);    overload;
  procedure AlertBx(const msg:string);                             overload;
  procedure AlertBx(const msg:string; const hlpctxt:THelpContext); overload;
  function  YesNobx(const msg:string):boolean;                                    overload;
  function  YesNobx(const msg,Yes,No:string):boolean;                             overload;
  function  YesNoBx(const msg,Yes,No:string; const hlpctxt:THelpContext):boolean; overload;
  function  StrInputBx(const title,msg,defaultval:string):string;                                 overload;
  function  StrInputBx(const title,msg,defaultval:string;const valType:TMdmValType):string;       overload;
  function  StrQueryBx(const title,msg:string; var val:string):boolean;                           overload;
  function  StrQueryBx(const title,msg:string; var val:string;const valType:TMdmValType):boolean; overload;
  function  StrBx(const title,msg:string; var val:string;const valType:TMdmValType):boolean;      overload;
  function  StrBx(const title,msg:string; var val:string):boolean;                                overload;
  function  MsgBx(const BoxMode:TmdMode;
                     const msg,b1caption,b2caption:string;
                     const hlpctxt:THelpContext):Boolean;

var
  fMessageDialogBx: TfMessageDialogBx;

implementation


{$R *.DFM}

procedure OkBx(const msg:string);
begin
  OkBx(msg,0);
end;

procedure OkBx(const msg:string; const hlpctxt:THelpContext);
begin
  MsgBxDlgExecute(mdmOK,msg,'Continue','',hlpctxt);
end;

procedure AlertBx(const msg:string);
begin
  AlertBx(msg,0);
end;

procedure AlertBx(const msg:string; const hlpctxt:THelpContext);
begin
  MsgBxDlgExecute(mdmAlert,msg,'OK','',hlpCtxt);
end;

function  YesNoBx(const msg:string):boolean;
begin
  result := YesNoBx(msg,'Yes','No',0);
end;

function  YesNoBx(const msg,Yes,No:string):boolean;
begin
  result := YesNoBx(msg,Yes,No,0);
end;

function  YesNoBx(const msg,Yes,No:string; const hlpctxt:THelpContext):boolean;
begin
  result := MsgBxDlgExecute(mdmQues,msg,Yes,No,hlpCtxt);
end;


//******** STRINPUTBX - Replaces InputBox ********************************
function  StrInputBx(const title,msg,defaultval:string):string;
begin
  result := strInputBx(title,msg,defaultval,mdmVTAnyStr);
end;

function  StrInputBx(const title,msg,defaultval:string;const valType:TMdmValType):string;       overload;
var
  v : string;
begin
  v := defaultval;
  MsgBxDlgExecute(mdmInput,Title+'|'+msg,'','',0,v,valtype);
  result := v;
end;



//******** STRQUERYBX - Replaces InputQuery ********************************
function  StrQueryBx(const title,msg:string; var val:string):boolean;
begin
  result := strQueryBx(title,msg,val,mdmVTAnyStr);
end;

function  StrQueryBx(const title,msg:string; var val:string;const valType:TMdmValType):boolean; overload;
begin
  result := MsgBxDlgExecute(mdmInput,Title+'|'+msg,'Ok','Cancel',0,val,valtype);
end;


//******** STRBX - Logical value return as StrQueryBx but without buttons **
function  StrBx(const title,msg:string; var val:string):boolean;
begin
  result := strBx(title,msg,val,mdmVTAnyStr);
end;

function  StrBx(const title,msg:string; var val:string;const valType:TMdmValType):boolean; overload;
begin
  result := MsgBxDlgExecute(mdmInput,Title+'|'+msg,'','',0,val,valtype);
end;

function  MsgBx(const BoxMode:TmdMode;
                const msg,b1caption,b2caption:string;
                const hlpctxt:THelpContext):Boolean;
begin
  result := MsgBxDlgExecute(BoxMode,msg,B1Caption,B2Caption,hlpCtxt);
end;




function MsgBxDlgExecute(const BoxMode:TmdMode;
                         const msg,b1caption,b2caption:string;
                         const hlpctxt:THelpContext):Boolean;
{ variant om master call without a var value }
var
  dummy:string;
begin
  result := MsgBxDlgExecute(BoxMode,msg,b1Caption,b2Caption,hlpCtxt,dummy);
end;


function MsgBxDlgExecute(const BoxMode:TmdMode;
                         const msg,b1caption,b2caption:string;
                         const hlpctxt:THelpContext;
                         var   value:string ):Boolean;
{ variant on master call with a unrestricted string value }
begin
  result := MsgBxDlgExecute(BoxMode,msg,b1Caption,b2Caption,hlpCtxt,value,mdmVTAnyStr);
end;

function MsgBxDlgExecute(const BoxMode:TmdMode;
                         const msg,b1caption,b2caption:string;
                         const hlpctxt:THelpContext;
                         var   value:string;
                         const valType:TMdmValType):Boolean;
{ ***************************************************************
  ******************* This is the master call. ******************
  ***************************************************************
  Because we have to create the form, then do twiddles, then get input then free
  all from scratch this is a wrapper for the form's own execute }
var
  mbox : TfMessageDialogBx;
begin
  mbox := TfMessageDialogBx.create(application);
  try
    result := mbox.execute(BoxMode,msg,b1caption,b2caption,hlpctxt,value,valtype)
  finally
    mbox.free;
  end;
end;


function TfMessageDialogBx.Pickstrval:string;
{ possibly extend eb.text to full item in picklist }
begin
  result := ExtendPick(eb.text);
end;


function TfMessageDialogBx.ExtendPick(const txt:string):string;
{ if we have a decent bit of text, see if we can match it
  more completely in the list of items to pick }
var
  plist,t,s : string;
  p,i : integer;
begin
  result := txt;        // if we can't expand uniquely then do nothing
  if result = '' then exit; // nothing can be done

  t := '|'+uppercase(txt);
  plist := uppercase('|'+fstartval);
  p := pos(t,plist);
  if p=0 then exit;        // no match of any sort

  if pos(t,copy(plist,p+1,999))<>0 then exit;  // ambigious so don't jump to conclusions

  s := copy(fstartval,p,999);
  i := pos('|',s);
  if i>0 then s := copy(s,1,i-1);
  result := s;
end;


function TfMessageDialogBx.NextStrVal:string;
{ jump to the next string value in the pick list }
var
  i : integer;
  s : string;
begin
  result := eb.text;
  s :='|'+uppercase(fstartval)+'|'+uppercase(fstartval);  // get picklist and allow to wrap
  i := pos('|'+uppercase(eb.text),s);                     // where are we in the list
  if i = 0 then exit;                                     // silly
  s := copy(s,i+length(eb.text)+2,999);                   // head of list is now next item
  result := chop(s,'|');                                  // so get that item
end;


function TfMessageDialogBx.IsInputOK:boolean;
{ tests the value of the input to see if it is acceptable }
var
  t : string;
  i : integer;
  f : single;
  d : tdatetime;
begin
  // deal with strings
  result := (fvaltype = mdmVTAnyStr);
  if result then exit;
  i := 0; {shut up compiler warning}
  f := 0; {shut up compiler warning}

  if fvaltype = mdmVTSomeStr then begin
    result := (trim(eb.text) <> '');
    exit;
  end;

  if fvaltype=mdmVTPickStr then begin
    t := pickstrval;
    result := pos(uppercase('|'+t+'|'),uppercase('|'+fstartval+'|'))<>0;
    exit;
  end;

  // try to interpret the input as integer
  t := eb.text;

  if fvaltype in [mdmVTint,mdmVTPosInt,mdmVTPoZint] then begin
    try
      i := strtoint(t);
      result := true;
    except
      result := false;
    end;
    if result then begin
      result := (fvaltype=mdmVTint)
             or ((fvaltype=mdmvtposint) and (i >  0))
             or ((fvaltype=mdmvtpozint) and (i >= 0));
    end;
    exit;
  end;

  // try to interpret input as float
  if fvaltype in [mdmVTFloat,mdmVTPosFloat,mdmVTPoZFloat] then begin
    try
      f := strtofloat(t);
      result := true;
    except
      result := false;
    end;
    if result then begin
      result := (fvaltype=mdmVTfloat)
             or ((fvaltype=mdmvtposfloat) and (f >  0))
             or ((fvaltype=mdmvtpozfloat) and (f >= 0));
    end;
    exit;
  end;


  // try to interpret as a date
  if fvaltype in [mdmVTDy,mdmVTPastDy,mdmVTFutureDy,mdmVTEarlierDy,mdmVTLaterDy] then begin
    t := uppercase(t);
    result := length(t)>0;
    if result then begin
      result := Decodeddmmyy(t,d);
      if (not result) or (fvaltype=mdmVTDy) then exit;
      result := ((fvaltype=mdmVTPastDy) and (d<=date))
             or ((fvaltype=mdmVTFutureDy) and (d>=date))
             or ((fvaltype=mdmVTEarlierDy) and (d<=fStartdate))
             or ((fvaltype=mdmVTLaterDy) and (d>=fStartdate));
    end;
    exit;
  end;

  // we should never drop through to here!


end;

function TfMessageDialogBx.Decodeddmmyy(const text:string;var d:tdatetime):boolean;
{ return true if we can interpret the string as a date.
  put the decoded result into d
  Allow e/b/n/t }
var
  s : string;
  dd,mm,yy : word;
begin
  s := uppercase(trim(text));
  // trap no input
  result := length(s)>0;
  if not result then exit;
  // look for e/n/b/t special values
  d := 1;
  If s[1]='B' then d := 0;
  If s[1]='E' then d := 1e20;
  If s[1]='T' then d := Date;
  If s[1]='N' then d := -1;
  result := (d <> 1);
  if result then exit;
  // we need a ddmmyy then
  result := length(s)=6;
  if not result then exit;
  dd := strtointdef(copy(s,1,2),999);
  mm := strtointdef(copy(s,3,2),999);
  yy := strtointdef(copy(s,5,2),999);
  result := not ((dd=999) or (mm=999) or (yy=999));
  if result then result := (dd>0) and (dd<=31) and (mm>0) and (mm<=12);
  if not result then exit;
  if yy >57 then yy := yy+1900 else yy := yy + 2000;
  try
    d := encodedate(yy,mm,dd);
  except
    result := false;
  end;
end;



function TfMessageDialogBx.Execute(const BoxMode:TmdMode;
                         const msg,b1caption,b2caption:string;
                         const hlpctxt:THelpContext;
                         var   value:string;
                         const valType:TMdmValType):Boolean;
var
  x,s : string;
  d : tdatetime;
  i,w,wmax,h,htot : integer;
begin

  timer.enabled := false;
  // get the arguments stored locally and the bits we need visible
  fmode := Boxmode;
  fmsg := msg;
  fvaltype := valtype;
  fStartVal := value;
  if fvaltype in [mdmVTEarlierDy,mdmVTLaterDy] then begin
    if decodeddmmyy(fStartval,d) then fstartdate := d else fstartdate := date;
  end;


  // inputs use the first line as caption
  if fmode = mdmInput then caption := chop(fmsg,'|');


  // calculate text dimensions
  wmax := screen.pixelsperinch*2;
  htot := 0;
  x := fmsg;
  while x <> '' do begin
    s := chop(x,'|');
    if s='' then s := 'Ay';               // vertical space for blank lines
    htot := htot + canvas.textheight(s);
    w := canvas.textwidth(s);
    if w > wmax then wmax := w;
  end;
  w := canvas.textwidth(b1caption);
  if w > wmax then wmax := w;
  w := canvas.textwidth(b2caption);
  if w > wmax then wmax := w;
  if fmode = mdmInput then begin
    if eb.width > wmax then wmax := eb.width;
  end;
  clientwidth := wmax + 40;


  // do we have an input box
  if fmode = mdmInput then begin
    eb.visible := true;
    activecontrol := eb;
    eb.top := htot+20;
    htot := eb.top + eb.height;
    eb.left := (clientwidth - eb.width) div 2;
  end else begin
    eb.visible := false;
  end;

  // button 1
  if (fmode = mdmInput) and (b1caption='') then begin
    // disable all buttons if plain input.
    bu1.visible := false;
    bu2.visible := false;
    h := htot + 10;
  end else begin
    bu1.visible := true;
    bu1.caption := b1Caption;
    if b2caption <> '' then begin
      if b1caption[1] <> b2caption[1] then bu1.caption := '&'+bu1.caption;
    end;

    bu1.top := htot + 20;

    if fmode=mdmInput then begin
      // special case of input where we put the two buttons the same width for neatness
      bu1.width := canvas.textwidth('&'+b2caption)+20;
    end else begin
      bu1.width := canvas.textwidth(bu1.caption)+20;
    end;
    bu1.left := (clientwidth - bu1.width) div 2;
    h := bu1.top+bu1.height + 10;

    if b2caption = '' then begin
      bu2.visible := false;
    end else begin
      bu2.top := h;
      bu2.visible := true;
      bu2.caption := b2caption;
      if b1caption[1] <> b2caption[1] then bu2.caption := '&'+bu2.caption;
      bu2.width := canvas.textwidth(bu2.caption)+20;
      bu2.left := (clientwidth - bu2.width) div 2;
      h := bu2.top+bu2.height + 10;
    end;
  end;

  if hlpctxt = 0 then begin
    buhelp.visible := false;
  end else begin
    buhelp.top := h;
    h := buhelp.top+buhelp.height + 10;
    buhelp.visible := true;
    fHelpContext := HlpCtxt;
    buhelp.width := canvas.textwidth(buhelp.caption)+20;
    buhelp.left := (clientwidth - buHelp.width) div 2;
  end;

  clientheight := h + 10;

  // save to last to ensure we have the buttons available for enabling etc
  if eb.visible then begin
    if fvaltype=mdmVTPickStr then begin
      i := pos('|',value);
      if i = 0 then i := length(value)+1; // trap silly
      if i = 1 then i := 2;  // trap silly
      eb.text := copy(value,1,i-1);
    end else begin
      eb.text := value;
    end;
  end;

  result := showmodal=mrYes;            // All that and now we can show the form

  // return the result with special processing for pickstring
  if fmode=mdmInput then begin
    if fvaltype=mdmVTPickStr then value := PickStrVal
                             else value := eb.text;   // return the value
  end;

end;



procedure TfMessageDialogBx.FormPaint(Sender: TObject);
var
  x,s : string;
  h : integer;
begin
  // print message text
  x := fmsg;
  h := 5;
  while x <> '' do begin
    s := chop(x,'|');
    if s <> '' then begin
      canvas.textout(10,h,s);
      h := h + canvas.textheight(s);
    end else begin
      h := h + canvas.textheight('Ay');   // blank lines get vertical space
    end;
  end;

end;

procedure TfMessageDialogBx.FormActivate(Sender: TObject);
begin
  case fmode of
    mdmOK : begin
              caption := 'Information';
              color := clWhite;
              font.color := clGreen;
            end;
    mdmAlert : begin
                 caption := 'Alert';
                 color := clyellow;
                 font.color := clBlack;
                 timer.enabled := true;
               end;
    mdmQues : begin
                caption := 'Choice';
                color := clWhite;
                font.color := clBlue;
              end;
    mdmInput: begin
                color := clSilver;
                font.color := clBlue;
              end;
  end;{ case}


  // do our own centralising on the screen
  top := (screen.height - height ) div 2;
  left := (screen.width - width ) div 2;



end;

procedure TfMessageDialogBx.TimerTimer(Sender: TObject);
begin
  if color = clyellow then color := clWhite else color := clYellow;
  paint;
end;



function TfMessageDialogBx.chop(var Mainstring : string; const Chopper : string):string;
var
  silly:boolean;
  p : integer;
begin
  // trap exceptional cases
  silly := false;
  if Mainstring = '' then begin
    result := '';
    silly := true;
  end;
  if Chopper = '' then begin
    result := Mainstring;
    mainstring := '';
    silly := true;
  end;
  if silly then exit;

  p := pos(Chopper,Mainstring);
  if p = 0 then begin       {not found so return rest}
    result := Mainstring;
    mainstring := '';
  end else begin
    result := copy(Mainstring,1,p-1);    {return LH as function result}
    p := p + length(chopper);            {skip chopper}
    if p > length(Mainstring) then Mainstring := ''  {trap terminal chopper}
    else Mainstring := copy(Mainstring,p,length(mainstring)-(p-1));  {lose LH from main string}
  end;
end;


procedure TfMessageDialogBx.buHelpClick(Sender: TObject);
begin
  Application.helpcontext(fhelpContext);
end;



procedure TfMessageDialogBx.ebKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin

  if key=vk_return then begin
    key := 0;
    if isinputok then modalresult := mrYes
                 else beep;
  end;

  if key=vk_escape then begin
    key := 0;
    modalresult := mrNo;
  end;

  if fvaltype = mdmVTPickStr then begin
    if key = vk_down then eb.text := nextstrval;
  end;


end;

procedure TfMessageDialogBx.ebChange(Sender: TObject);
{ call to test the input to see if it is valid
  if not then highlight and possibly disable the OK button }
var
  p : string;
begin
  if fvaltype = mdmVTPickStr then begin
    p := pickstrval;
    if eb.text <> p then begin
      eb.text := p;       // this will cause a cascade but don't worry (only twice)
    end;
  end;

  if IsInputOK then begin
    eb.color := clwindow;
    if bu1.visible then bu1.enabled := true;
  end else begin
    eb.color := clYellow;
    if bu1.visible then bu1.enabled := false;
  end;
end;

procedure TfMessageDialogBx.ebKeyPress(Sender: TObject; var Key: Char);
var
  s : string;
begin
  // we may be using short cuts to pick a string
  if fvaltype = mdmVTPickStr then begin
    s := Extendpick(key);
    if s <> key then begin
      eb.text := s;
      key := #0;
    end;
  end;

  // we may be using short cuts to set the date
  if fvaltype in [mdmVTDy,mdmVTPastDy,mdmVTFutureDy,mdmVTEarlierDy,mdmVTLaterDy] then begin
    case pos(uppercase(key),'BETN') of
    0 : s := ''; //do nothing;
    1 : s := 'Beg-of-time';
    2 : s := 'End-of-time';
    3 : s := 'Today';
    4 : s := 'No date';
    end;{case}
    if s <> '' then begin
      eb.text := s;
      key := #0;
    end;
  end;
end;



(*   POSTSCRIPT
   
     Making your own dialogs      - 2 minute tutorial                                    
   

   (1) Hack together a form and test basic operation.
       A blank form will do for starters.

   (2) Add a function in the form's unit, but NOT a method of it which does
       function ShowDlg(const message:string):tModalresult;
       var
         dlg : TMyDlg;
       begin
         dlg := TMyDlg.create(application);
         try
           dlg.caption := message;        // illustrates some setting up routine
           result := dlg.showmodal;       // show form and get how the form was closed
           // or you could have a more complex analysis routine to work out an answer
         finally
           dlg.free;
         end;
       end;

   (3) You can now call ShowDlg FROM ANOTHER FORM...
         procedure TForm1.Button1Click(Sender: TObject);
         begin
           Showdlg('Hello world');
         end;  
       (a) The *other form* will be the main unit  (Project|Options|Forms...)
       (b) You'll have to put the dlg unit into the main form's uses clause


   (4) This doesn't make the dialog suitable for the component pallette.
       I'm happy with that but if you really want to drag and drop at design time
       then you have to jump through more hoops.  I believe you can't just
       creating a new component as a subclass of Tform and the get-round
       involves getting it working as a form then converting to a Tcomponent
       and doing the setup that would be done using the .dfm in code.  Then
       of course you have to package/install it.

   

   MsgBxDlgExecute is the master call as described in (2) and all the other
   calls feed into it.  In turn that calls an .execute method in the form
   which deals with the setup.  That may be too many levels of complexity for
   your needs.


*)



end.
